@@ -19,12 +19,11 @@ import gtk
 _LANDSCAPE_ARRANGEMENT = """
   <arrangement>
     <if-visible name="toolbar">
-      <widget name="btn_nav" x1="0" y1="-64" x2="64" y2="100%"/>
       <widget name="toolbar" x1="-80" y1="0" x2="100%" y2="100%"/>
 
       <widget name="screen" x1="40" y1="4" x2="-84" y2="-64"/>
-      <widget name="progress" x1="90" y1="-50" x2="-154" y2="-10"/>
-      <widget name="btn_star" x1="-154" y1="-64" x2="-90" y2="100%"/>
+      <widget name="progress" x1="80" y1="-50" x2="-90" y2="-10"/>
+      <widget name="btn_star" x1="10" y1="-64" x2="74" y2="100%"/>
       <widget name="slider" x1="0" y1="0" x2="40" y2="-64"/>
     </if-visible>
     
@@ -36,11 +35,10 @@ _LANDSCAPE_ARRANGEMENT = """
 """
 
 # hmm, there is no such thing as portrait mode video. Fremantle either does not
-# rotate or will just crash, depending on firmware version
+# rotate or will just crash, depending on the firmware version
 # we simply hide the video screen while in portrait mode and play sound only
 _PORTRAIT_ARRANGEMENT = """
   <arrangement>
-    <widget name="btn_nav" x1="-64" y1="0" x2="100%" y2="64"/>
     <widget name="toolbar" x1="0" y1="-80" x2="100%" y2="100%"/>
 
     <widget name="screen" x1="40" y1="0" x2="100%" y2="-80"/>
@@ -58,7 +56,6 @@ class VideoPlayer(Player):
     def __init__(self):
 
         self.__is_fullscreen = False
-        self.__is_portrait = False
         self.__volume = 50
 
         self.__player = None
@@ -99,13 +96,6 @@ class VideoPlayer(Player):
                                       theme.mb_btn_bookmark_2)
         self.__btn_star.connect_clicked(self.__on_btn_star)
 
-
-        # navigator button
-        self.__btn_navigator = ImageButton(theme.mb_btn_navigator_1,
-                                           theme.mb_btn_navigator_2)
-        self.__btn_navigator.connect_clicked(
-             lambda *a:self.emit_message(msgs.UI_ACT_SHOW_DIALOG, "navigator.Navigator"))
-
         
         # toolbar elements
         self.__btn_play = ImageButton(theme.mb_btn_play_1,
@@ -132,7 +122,6 @@ class VideoPlayer(Player):
         self.__arr.connect_resized(self.__update_layout)
         self.__arr.add(self.__screen, "screen")
         self.__arr.add(self.__toolbar, "toolbar")
-        self.__arr.add(self.__btn_navigator, "btn_nav")
         self.__arr.add(self.__progress, "progress")
         self.__arr.add(self.__btn_star, "btn_star")
         self.__arr.add(self.__volume_slider, "slider")
@@ -143,7 +132,8 @@ class VideoPlayer(Player):
     
         w, h = self.get_size()
         if (w < h):
-            self.__btn_star.set_visible(False)
+            if (not self.__is_fullscreen):
+                self.__btn_star.set_visible(False)
             self.__arr.set_xml(_PORTRAIT_ARRANGEMENT)           
         else:
             self.__btn_star.set_visible(True)
@@ -176,8 +166,6 @@ class VideoPlayer(Player):
 
     def __on_tap(self, px, py):
     
-        #if (self.__player):
-        #    self.__player.pause()
         self.__toggle_fullscreen()
 
 
@@ -299,12 +287,12 @@ class VideoPlayer(Player):
             self.__progress.set_visible(False)
             self.__volume_slider.set_visible(False)
             self.__toolbar.set_visible(False)
-            self.__btn_navigator.set_visible(False)
+            self.__btn_star.set_visible(False)
         else:
             self.__progress.set_visible(True)
             self.__volume_slider.set_visible(True)
             self.__toolbar.set_visible(True)
-            self.__btn_navigator.set_visible(True)
+            self.__btn_star.set_visible(True)
 
         self.emit_message(msgs.UI_ACT_FULLSCREEN, self.__is_fullscreen)
         self.__update_layout()
@@ -322,11 +310,14 @@ class VideoPlayer(Player):
         screen.fill_area(x, y, w, h, theme.color_mb_background)
         self.__arr.set_geometry(0, 0, w, h)
         
-        if (self.__is_portrait):
+        if (w < h):
+            self.__screen.set_visible(False)
             screen.draw_centered_text("Video cannot be displayed in\n" \
                                       "portrait mode.",
                                       theme.font_mb_plain,
                                       0, h / 2 - 80, w, 0, theme.color_mb_text)
+        else:
+            self.__screen.set_visible(True)
 
         
     def get_mime_types(self):
@@ -394,7 +385,7 @@ class VideoPlayer(Player):
 
     def handle_MEDIA_ACT_PLAY(self):
     
-        if (self.__player and self.is_visible()):
+        if (self.__player and self.is_player_active()):
             if (not self.__is_playing):
                 self.__wait_for_dsp()
             self.__player.play()
@@ -402,29 +393,26 @@ class VideoPlayer(Player):
 
     def handle_MEDIA_ACT_PAUSE(self):
     
-        if (self.__player and self.is_visible()):
+        if (self.__player and self.is_player_active()):
             if (not self.__is_playing):
                 self.__wait_for_dsp()
-            self.__player.pause()
-                
+            self.__player.pause()                
 
 
     def handle_MEDIA_ACT_STOP(self):
     
-        if (self.__player and self.is_visible()):
+        if (self.__player and self.is_player_active()):
             self.__player.stop()
 
 
     def handle_ASR_EV_LANDSCAPE(self):
 
-        self.__is_portrait = False
         self.__screen.set_visible(True)
         self.render()
         
-        
+    """
     def handle_ASR_EV_PORTRAIT(self):
 
-        self.__is_portrait = True
         self.__screen.set_visible(False)
         self.render()
         
@@ -433,6 +421,7 @@ class VideoPlayer(Player):
 
         if (self.is_visible()):
             self.__toggle_fullscreen()
+    """
 
 
     def handle_INPUT_EV_VOLUME_UP(self, pressed):
