@@ -234,14 +234,14 @@ class AbstractBackend(EventEmitter):
     
         if (self.__suspension_point):
             uri, pos = self.__suspension_point
-            self.__suspension_point = None
+            #self.__suspension_point = None
 
             self._ensure_backend()
 
             self._load(uri)
             #self._set_volume(self.__volume)
-            gobject.idle_add(self._seek, pos)
-            self._stop()
+            #gobject.idle_add(self._seek, pos)
+            #self._stop()
             self.__position = (0, 0)
             self.__watch_progress()
             
@@ -273,11 +273,20 @@ class AbstractBackend(EventEmitter):
 
         if (self.__playing):
             pos, total = self.__position
+ 
+            if (self.__suspension_point and pos > 0):
+                # seek to suspension point
+                url, pos = self.__suspension_point
+                self.__suspension_point = None
+                self._seek(pos)
+            #end if
+
             if (pos < 3):
                 pos, total = self._get_position()
                 timestamp = time.time()
                 beginpos = pos
             else:
+            
                 # we don't ask the backend for position every time because
                 # this could be inefficient with some backends
                 pos = beginpos + (time.time() - timestamp)
@@ -289,7 +298,7 @@ class AbstractBackend(EventEmitter):
                                     self.__context_id, pos, total)
 
             if (total > 0 and total - pos < 1):
-                delay = 200
+                delay = int(total - pos) * 1000 #200
             else:
                 delay = 500
             self.__position_handler = \
@@ -406,7 +415,7 @@ class AbstractBackend(EventEmitter):
         #self._set_volume(self.__volume)
         
         print "DELAY PLAY", uri
-        gobject.timeout_add(0, self.play)
+        gobject.idle_add(self.play)
 
         if (ctx_id != -1):
             self.__context_id = ctx_id
@@ -476,7 +485,11 @@ class AbstractBackend(EventEmitter):
         self.stop()
         self.emit_event(self.EVENT_ERROR, self.__context_id, err)
         self.__eof_reached = True
-       
+      
+      
+    def get_position(self):
+    
+        return self.__position 
         
         
     def play(self):
@@ -491,7 +504,6 @@ class AbstractBackend(EventEmitter):
             self._play()
 
         self.__playing = True
-        print "PLAY"
         self.emit_event(self.EVENT_STATUS_CHANGED,
                         self.__context_id, self.STATUS_PLAYING)
         self.__watch_progress()
