@@ -74,7 +74,7 @@ class Window(Widget):
                 self.__window.set_app_menu(self.__menu)
             elif (platforms.MAEMO4):
                 self.__window = hildon.Window()
-                self.__window.set_menu(self.__menu)
+                #self.__window.set_menu(self.__menu)
                 self.__window.fullscreen()
             elif (platforms.MEEGO_NETBOOK):
                 self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
@@ -86,12 +86,14 @@ class Window(Widget):
 
         elif (wtype == self.TYPE_SUBWINDOW):
             if (platforms.MAEMO5):
-                self.__window = gtk.Dialog()                
+                self.__window = hildon.StackableWindow()
+                self.__window.set_app_menu(self.__menu)
             elif (platforms.MAEMO4):
-                self.__window = gtk.Dialog()
-                self.__window.set_decorated(False)
+                self.__window = gtk.Window()
+                #self.__window.set_decorated(False)
+                self.__window.fullscreen()
                 # hide some ugly separator :)
-                self.__window.vbox.get_children()[0].hide()
+                #self.__window.vbox.get_children()[0].hide()
             elif (platforms.MEEGO_NETBOOK):
                 self.__window = gtk.Dialog()
                 self.__window.set_decorated(False)
@@ -100,9 +102,9 @@ class Window(Widget):
                 # hide some ugly separator :)
                 self.__window.vbox.get_children()[0].hide()                
             else:
-                self.__window = gtk.Dialog()
+                self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                 # hide some ugly separator :)
-                self.__window.vbox.get_children()[0].hide()
+                #self.__window.vbox.get_children()[0].hide()
 
                             
         elif (wtype == self.TYPE_DIALOG):
@@ -134,7 +136,7 @@ class Window(Widget):
             self.__title_bar = None
             
         self.__window.set_title(" ")
-        self.__window.set_default_size(800, 480)
+        self.__window.set_default_size(480, 800) #800, 480)
         self.__window.set_app_paintable(True)
         self.__window.set_double_buffered(False)
 
@@ -157,7 +159,7 @@ class Window(Widget):
         self.__window.realize()
 
         if (platforms.MAEMO5):
-            if (wtype == self.TYPE_TOPLEVEL):
+            if (wtype in [self.TYPE_TOPLEVEL, self.TYPE_SUBWINDOW]):
                 pass #self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
             else:
                 self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
@@ -182,7 +184,7 @@ class Window(Widget):
                                     gtk.gdk.KEY_RELEASE_MASK)
         self.__layout.put(self.__vidscreen, 0, 0)
 
-        gobject.timeout_add(0, self.__update_window_background)
+        self.__update_window_background()
 
 
     def connect_closed(self, cb, *args):
@@ -301,12 +303,22 @@ class Window(Widget):
 
 
     def __on_configure(self, src, ev):
-
-        if (self.__configure_event_handler):
-            gobject.source_remove(self.__configure_event_handler)
-        self.__configure_event_handler = gobject.timeout_add(0, 
-                                                self.__handle_configure_event)
-
+        
+        w = ev.width
+        h = ev.height
+        if ((w, h) != self.__size):
+            self.__size = (w, h)
+            if (self.__configure_event_handler):
+                gobject.source_remove(self.__configure_event_handler)
+            else:
+                # freeze rendering at the first of possibly several configure events
+                # in a row
+                self.set_frozen(True)
+                
+            self.__configure_event_handler = gobject.timeout_add(100,
+                                                 self.__handle_configure_event)
+        #end if
+        
 
     def __handle_configure_event(self):
         """
@@ -314,16 +326,18 @@ class Window(Widget):
         shoots several in a row and we should only react on the last one.
         """
 
+        # thaw rendering at the last of possibly several configure events in
+        # a row
+        self.set_frozen(False)
+
         self.__configure_event_handler = None
         w, h = self.__window.get_size()
-        if ((w, h) != self.__size):
-            self.__screen = Pixmap(self.__window.window)
-            self.set_screen(self.__screen)
-            self.__size = (w, h)
+        self.__screen = Pixmap(self.__window.window)
+        self.__screen.fill_area(0, 0, w, h, theme.color_mb_background)
+        self.set_screen(self.__screen)
 
-            self.set_size(w, h)
-            self.render()
-        #end if
+        self.set_size(w, h)
+        self.render()
 
 
     def __on_expose(self, src, ev):
@@ -494,7 +508,7 @@ class Window(Widget):
             elif (platforms.MAEMO4):
                 if (self.__wtype != self.TYPE_TOPLEVEL and self.__has_size_set):
                     w, h = self.__window.get_size()
-                    self.__window.move(0, 480 - h)
+                    self.__window.move(0, 480 - max(h, 400))
 
             #endif
 
