@@ -8,7 +8,7 @@ import time
 
 
 # hide the letter display after this many milliseconds without scrolling
-_LETTER_TIMEOUT = 100
+_LETTER_TIMEOUT = 500
 
 # letters are only displayed if the list is at least this many times bigger
 # than the screen size
@@ -22,9 +22,19 @@ class ThumbableGridView(GridView):
     
 
     def __init__(self):
-    
-        # the associated slider
-        self.__slider = None
+       
+        # value of the built-in slider
+        self.__slider_value = 0.0
+        
+        # slider grab position
+        self.__slider_grab_pos = 0
+        
+        # whether the user is using the slider
+        self.__is_sliding = False
+       
+        
+        # whether multi select is enabled
+        self.__multi_select = False
                
         # whether the index letter is enabled
         self.__letter_enabled = True
@@ -44,12 +54,15 @@ class ThumbableGridView(GridView):
         # click behavior for selecting items
         self.__click_behavior = self.CLICK_BEHAVIOR_SINGLE
         
+        # index of the marked item
+        self.__marked_idx = -1
         
         GridView.__init__(self)
         self.connect_button_pressed(self.__on_press_button)
         self.connect_button_released(self.__on_release_button)
         self.connect_pointer_moved(self.__on_pointer_moved)
         self.add_overlay_renderer(self.__render_letter)
+        self.add_overlay_renderer(self.__render_slider)
         
         self.__kscr = KineticScroller(self)
         self.__kscr.connect_scrolling_started(self.__on_scrolling_started)
@@ -89,6 +102,15 @@ class ThumbableGridView(GridView):
         #end if
 
 
+    def __render_slider(self, screen):
+    
+        w, h = self.get_size()
+        t_w, t_h = self.get_total_size()
+        if (self.__letter_visible and t_h > h * 2):
+            h -= theme.mb_list_slider.get_height()
+            screen.draw_pixbuf(theme.mb_list_slider, 0, self.__slider_value * h)
+
+
     def __on_scrolling_started(self):
     
         #csr = self.get_cursor()
@@ -110,37 +132,35 @@ class ThumbableGridView(GridView):
         csr = self.get_cursor()
         item = self.get_item(idx)
 
-        # the touchscreen of Maemo4 devices is not sensitive enough for
-        # single click operation
-        if (self.__click_behavior == self.CLICK_BEHAVIOR_SINGLE or
-            (idx == self.get_cursor() and
-             time.time() - self.__last_cursor_time > 0.5)):
-            item.click_at(px - x, py - y)
-            self.set_cursor(-1)
-            self.invalidate_item(idx)
-        else:
-            self.set_cursor(idx)
-            self.__last_cursor_time = time.time()
-            if (csr != -1):
-                self.invalidate_item(csr)
-            self.invalidate_item(idx)
-            #self.render()    
+        if (self.__multi_select):
+            item.set_selected(not item.is_selected())
 
+        elif (px > 40):
+            # the touchscreen of Maemo4 devices is not sensitive enough for
+            # single click operation
+            if (self.__click_behavior == self.CLICK_BEHAVIOR_SINGLE or
+                (idx == self.get_cursor() and
+                 time.time() - self.__last_cursor_time > 0.5)):
+                item.click_at(px - x, py - y)
+                self.set_cursor(-1)
+            else:
+                self.set_cursor(idx)
+                self.__last_cursor_time = time.time()
+                if (csr != -1):
+                    self.invalidate_item(csr)
+                #self.render()
+            #end if
+        #end if
+        self.invalidate_item(idx)
+        
         
     def __on_tap_and_hold(self, px, py):
 
-        if (not self.has_floating_item()):
+        if (not self.__multi_select and not self.has_floating_item()):
             idx = self.get_item_at(px, py)
             x, y = self.get_position_of_item(idx)
             item = self.get_item(idx)
             item.tap_and_hold(px - x, py - y)
-
-            #csr = self.get_cursor()
-            #if (csr != -1):
-            #    self.set_cursor(idx)
-            #    self.invalidate_item(csr)
-            #    self.render()
-            #end if
         #end if
 
 
@@ -149,7 +169,7 @@ class ThumbableGridView(GridView):
         idx = self.get_item_at(px, py)
         item = self.get_item(idx)
         w, h = self.get_size()
-        
+
         if (px > w - 80 and item.is_draggable()):
             # handle drag-sorting
             self.float_item(idx, px - w, py)
@@ -157,11 +177,24 @@ class ThumbableGridView(GridView):
             self.invalidate()
             self.render()
 
+        elif (px < 40):
+            self.stop_scrolling()
+            self.__is_sliding = True
+            w, h = self.get_size()
+            h -= theme.mb_list_slider.get_height()
+            self.__slider_grab_pos = py - self.__slider_value * h
+            
         else:
             # handle clicking
             if (item.is_button()):
+                if (self.__marked_idx != -1):
+                    item2 = self.get_item(self.__marked_idx)
+                    item2.set_marked(False)
+                    self.invalidate_item(item2)
+                    
                 item.set_marked(True)
                 self.invalidate_item(idx)
+                self.__marked_idx = idx
         
 
 
@@ -174,12 +207,18 @@ class ThumbableGridView(GridView):
 
             self.render()
 
+        elif (self.__is_sliding):
+            self.stop_scrolling()
+            self.__is_sliding = False
+
         else:
             idx = self.get_item_at(px, py)
             item = self.get_item(idx)
             if (item.is_button()):
-                item.set_marked(False)
-                self.invalidate_item(idx)
+                item2 = self.get_item(self.__marked_idx)
+                item2.set_marked(False)
+                self.invalidate_item(self.__marked_idx)
+                self.__marked_idx = -1
 
         if (self.__autoscroll_handler):
             gobject.source_remove(self.__autoscroll_handler)
@@ -214,19 +253,19 @@ class ThumbableGridView(GridView):
                 self.__autoscroll_handler = gobject.timeout_add(
                                                  50, self.__do_autoscroll, 30)
 
+        elif (self.__is_sliding):
+            w, h = self.get_size()
+            total_w, total_h = self.get_total_size()
+            percent = (py - self.__slider_grab_pos) / \
+                      float((h - theme.mb_list_slider.get_height()))
+            percent = max(min(percent, 1.0), 0.0)
+            prev_offset = self.get_offset()
+            offset = int((total_h - h) * percent)
+            #self.stop_scrolling()
+            self.move(0, offset - prev_offset)
         #end if
 
 
-        
-    def __on_drag_slider(self, percent):
-
-        w, h = self.get_size()
-        total_w, total_h = self.get_total_size()
-        prev_offset = self.get_offset()
-        offset = int((total_h - h) * (1.0 - percent))
-        self.move(0, offset - prev_offset)
-
-
     def __on_letter_timeout(self):
     
         self.__letter_timeout_handler = None
@@ -267,6 +306,33 @@ class ThumbableGridView(GridView):
         return (dx, dy)
 
 
+    def set_multi_select(self, value):
+    
+        self.__multi_select = value
+        for item in self.get_items():
+            item.set_selected(False)
+        self.invalidate()
+        self.render()
+
+
+    def select_all(self):
+        
+        if (self.__multi_select):
+            for item in self.get_items():
+                item.set_selected(True)
+            self.invalidate()
+            self.render()
+        #end if            
+
+
+    def unselect_all(self):
+        
+        for item in self.get_items():
+            item.set_selected(False)
+        self.invalidate()
+        self.render()
+
+
     def set_letter_enabled(self, value):
     
         self.__letter_enabled = value
@@ -290,6 +356,15 @@ class ThumbableGridView(GridView):
 
     def __update_slider(self):
 
+        w, h = self.get_size()
+        total_w, total_h = self.get_total_size()
+        if (total_h <= h):
+            percent = 0.0
+        else:
+            percent = self.get_offset() / float(total_h - h)
+        self.__slider_value = percent
+
+        """
         if (self.__slider):
             w, h = self.get_size()
             total_w, total_h = self.get_total_size()
@@ -299,20 +374,8 @@ class ThumbableGridView(GridView):
                 percent = self.get_offset() / float(total_h - h)
             self.__slider.set_value(1.0 - percent)
         #end if    
-    
-
-        
-    def associate_with_slider(self, slider):
         """
-        Associates this item view with a slider widget. Only one slider may be
-        associated with the item view at a time.
-        
-        @param slider: slider widget
-        """
-    
-        self.__slider = slider
-        self.__slider.connect_value_changed(self.__on_drag_slider)
-        
+       
         
     def connect_item_activated(self, cb, *args):
         
