@@ -1,11 +1,11 @@
 import ctypes
 import os
 
-_xlib = ctypes.CDLL("libx11.so.6")
+_xlib = ctypes.CDLL("libX11.so.6")
 _xrandr = ctypes.CDLL("libXrandr.so.2")
+rr = _xrandr
 
-
-class _XRRScreenConfiguration(Structure): pass
+class _XRRScreenConfiguration(ctypes.Structure): pass
 
 
 def set_orientation(orientation):
@@ -15,7 +15,7 @@ def set_orientation(orientation):
     root = _xlib.XDefaultRootWindow(display, screen)
 
     gsi = rr.XRRGetScreenInfo
-    gsi.restype = ctypes.POINTER(XRRScreenConfiguration)
+    gsi.restype = ctypes.POINTER(_XRRScreenConfiguration)
     config = gsi(display, root)
 
     current_time = ctypes.c_ulong()
